package mapreduce

import org.scalatest._
import scala.util.Random

class Test extends FlatSpec {  
  
  "moreThanThreshold" should "retain exactly the elements that are greater than the threshold" in {
      def generate : (List[Int], Int, List[Int] ) = {
        val c = Random.nextInt(20)
        def generate(list : List[Int], result : List[Int]) : (List[Int], Int, List[Int] ) = Random.nextInt(5) match {
          case 0 => (list, c, result)
          case _ => {
            val r = Random.nextInt(20)            
            if( r > c ) generate( r :: list , r :: result ) else generate( r :: list , result ) 
          }
        }
        generate( Nil, Nil )        
      }
      for (i <- 1 to 1000) {
        val (list,c,result) = generate
        val tested = UtilityObject.moreThanThreshold( list, c ) 
        assert( tested === result,
            "\nList is: " + list.mkString(",") + "\nThreshold is: " + c + "\nShould return: " + result + "\nReturned: " + tested )
        
      }
  }
  
  "projectLeft" should "project to the left coordinate" in {
      def generate( list: List[(String,Int)], result : List[String] ) : (List[(String,Int)], List[String] ) =
      Random.nextInt(5) match {
        case 0 => (list,result)
        case _ => {
          val (newName,newInt) = (Random.nextPrintableChar().toString(), Random.nextInt(20) )
          generate( (newName,newInt) :: list , newName :: result )
        }
      }
      for( i <- 1 to 1000 ) {
        val (list, result) = generate( Nil,Nil )
        val tested = UtilityObject.projectLeft( list )
        assert( tested === result,
            "\nList is: " + list + "\nShould return: " + result + "\nReturned: " + tested )        
     }
  }
  
  "nameMoreThanThreshold" should "project to the left coordinate those members having larger data than the given threshold" in {
   def generate( list: List[(String,Int)], c : Int, result : List[String] ) : (List[(String,Int)], Int, List[String] ) =
      Random.nextInt(5) match {
        case 0 => (list,c,result)
        case _ => {
          val (newName,newInt) = (Random.nextPrintableChar().toString(), Random.nextInt(20) )
          if( newInt > c ) generate( (newName,newInt) :: list , c, newName :: result ) else generate( (newName,newInt) :: list , c, result )
        }
      }
      for( i <- 1 to 1000 ) {
        val (list, c, result) = generate( Nil, Random.nextInt( 20 ) , Nil )
        val tested = UtilityObject.nameMoreThanThreshold(list, c)
        assert( tested === result,
            "\nList is: " + list + "\nThreshold is: " + c + "\nShould return: " + result + "\nReturned: " + tested )        
     } 
  }
  
  it should "work for empty input" in {
    assert( UtilityObject.nameMoreThanThreshold(List(), 10) === List() )
  }
  
  "sumRight" should "work for empty input" in {
    assert( UtilityObject.sumRight( Nil ) === 0 )
  }
  
  it should "sum the second coordinates" in {
    def generate( list : List[(String,Int)], result : Int ) : (List[(String,Int)], Int )= Random.nextInt(5) match {
      case 0 => { val a = Random.nextInt(20) ; (( Random.nextPrintableChar().toString() , a ) :: list , result + a ) }
      case _ => {
        val a = Random.nextInt( 20 )
        generate( ( Random.nextPrintableChar().toString(), a ) :: list , result + a )
      }
    }
    for( i <- 1 to 1000 ) {
      val (list, result) = generate( Nil, 0 )
      val tested = UtilityObject.sumRight( list )
      assert( result === tested ,
       "\nList is: " + list + "\nShould return: " + result + "\nReturned: " + tested    
      )
    }
  }
  
  "average" should "work for empty input" in {
    assert( UtilityObject.average( Nil ).isNaN() )
  }
  
  it should "calculate the average" in {
    def generate( list : List[ Int ], sum : Int, count : Int ) : (List[Int], Double ) = Random.nextInt(5) match {
      case 0 => (list, sum.toDouble / count )
      case _ => {
        val a = Random.nextInt( 20 )
        generate( a :: list , a + sum, count + 1 )
      }
    }
    for( i <- 1 to 1000 ) {
      val (list, result) = { val a = Random.nextInt(20); generate( List(a), a, 1 ) }
      val tested = UtilityObject.average( list )
      assert( result === tested ,
        "\nList is: " + list + "\nShould return: " + result + "\nReturned: " + tested    
      )  
    }
  }
  
  "printLargeStuff" should "work for empty input" in {
    assert( UtilityObject.printLargeStuff( Nil ) === Nil )
  }
  
  it should "print out the names of those guys earning more than 1.5x the average" in {
  def generate( list : List[ (String,Int) ], sum : Int, count : Int ) : (List[(String,Int)], Double ) = Random.nextInt(5) match {
      case 0 => (list, sum.toDouble / count )
      case _ => {
        val a = Random.nextInt( 20 )
        generate( (Random.nextPrintableChar().toString(), a) :: list , a + sum, count + 1 )
      }
    }
    for( i <- 1 to 1000 ) {
      val (list, avg) = { val a = Random.nextInt(20); generate( List((Random.nextPrintableChar().toString(), a)), a, 1 ) }
      val result = list filter { _._2 > 1.5*avg } map { _._1 }
      val tested = UtilityObject.printLargeStuff( list )
      assert( result === tested ,
        "\nList is: " + list + "\nShould return: " + result + "\nReturned: " + tested    
      )  
    }  
  }
}